# -*- coding: utf-8 -*-
import inspect
import os.path as op  # system functions

from ....interfaces import io as nio  # Data i/o
from ....interfaces import utility as util  # utility
from ....pipeline import engine as pe  # pypeline engine
from ....interfaces import fsl as fsl
from ....interfaces import freesurfer as fs  # freesurfer
from ....interfaces import mrtrix as mrtrix
from ....interfaces import cmtk as cmtk
from ....interfaces import dipy as dipy
from ....algorithms import misc as misc
from ..fsl.epi import create_eddy_correct_pipeline
from ..connectivity.nx import create_networkx_pipeline, create_cmats_to_csv_pipeline
from ....interfaces.utility import Function
from ...misc.utils import select_aparc_annot


def create_connectivity_pipeline(name="connectivity",
                                 parcellation_name='scale500'):
    """Creates a pipeline that does the same connectivity processing as in the
    :ref:`example_dmri_connectivity_advanced` example script. Given a subject id (and completed Freesurfer reconstruction)
    diffusion-weighted image, b-values, and b-vectors, the workflow will return the subject's connectome
    as a Connectome File Format (CFF) file for use in Connectome Viewer (http://www.cmtk.org).

    Example
    -------

    >>> from nipype.workflows.dmri.mrtrix.connectivity_mapping import create_connectivity_pipeline
    >>> conmapper = create_connectivity_pipeline("nipype_conmap")
    >>> conmapper.inputs.inputnode.subjects_dir = '.'
    >>> conmapper.inputs.inputnode.subject_id = 'subj1'
    >>> conmapper.inputs.inputnode.dwi = 'data.nii.gz'
    >>> conmapper.inputs.inputnode.bvecs = 'bvecs'
    >>> conmapper.inputs.inputnode.bvals = 'bvals'
    >>> conmapper.run()                 # doctest: +SKIP

    Inputs::

        inputnode.subject_id
        inputnode.subjects_dir
        inputnode.dwi
        inputnode.bvecs
        inputnode.bvals
        inputnode.resolution_network_file

    Outputs::

        outputnode.connectome
        outputnode.cmatrix
        outputnode.networks
        outputnode.fa
        outputnode.struct
        outputnode.tracts
        outputnode.rois
        outputnode.odfs
        outputnode.filtered_tractography
        outputnode.tdi
        outputnode.nxstatscff
        outputnode.nxcsv
        outputnode.cmatrices_csv
        outputnode.mean_fiber_length
        outputnode.median_fiber_length
        outputnode.fiber_length_std
    """

    inputnode_within = pe.Node(
        util.IdentityInterface(fields=[
            "subject_id", "dwi", "bvecs", "bvals", "subjects_dir",
            "resolution_network_file"
        ]),
        name="inputnode_within")

    FreeSurferSource = pe.Node(
        interface=nio.FreeSurferSource(), name='fssource')
    FreeSurferSourceLH = pe.Node(
        interface=nio.FreeSurferSource(), name='fssourceLH')
    FreeSurferSourceLH.inputs.hemi = 'lh'

    FreeSurferSourceRH = pe.Node(
        interface=nio.FreeSurferSource(), name='fssourceRH')
    FreeSurferSourceRH.inputs.hemi = 'rh'
    """
    Creating the workflow's nodes
    =============================
    """
    """
    Conversion nodes
    ----------------
    """
    """
    A number of conversion operations are required to obtain NIFTI files from the FreesurferSource for each subject.
    Nodes are used to convert the following:
        * Original structural image to NIFTI
        * Pial, white, inflated, and spherical surfaces for both the left and right hemispheres are converted to GIFTI for visualization in ConnectomeViewer
        * Parcellated annotation files for the left and right hemispheres are also converted to GIFTI

    """

    mri_convert_Brain = pe.Node(
        interface=fs.MRIConvert(), name='mri_convert_Brain')
    mri_convert_Brain.inputs.out_type = 'nii'
    mri_convert_ROI_scale500 = mri_convert_Brain.clone(
        'mri_convert_ROI_scale500')

    mris_convertLH = pe.Node(interface=fs.MRIsConvert(), name='mris_convertLH')
    mris_convertLH.inputs.out_datatype = 'gii'
    mris_convertRH = mris_convertLH.clone('mris_convertRH')
    mris_convertRHwhite = mris_convertLH.clone('mris_convertRHwhite')
    mris_convertLHwhite = mris_convertLH.clone('mris_convertLHwhite')
    mris_convertRHinflated = mris_convertLH.clone('mris_convertRHinflated')
    mris_convertLHinflated = mris_convertLH.clone('mris_convertLHinflated')
    mris_convertRHsphere = mris_convertLH.clone('mris_convertRHsphere')
    mris_convertLHsphere = mris_convertLH.clone('mris_convertLHsphere')
    mris_convertLHlabels = mris_convertLH.clone('mris_convertLHlabels')
    mris_convertRHlabels = mris_convertLH.clone('mris_convertRHlabels')
    """
    Diffusion processing nodes
    --------------------------

    .. seealso::

        dmri_mrtrix_dti.py
            Tutorial that focuses solely on the MRtrix diffusion processing

        http://www.brain.org.au/software/mrtrix/index.html
            MRtrix's online documentation
    """
    """
    b-values and b-vectors stored in FSL's format are converted into a single encoding file for MRTrix.
    """

    fsl2mrtrix = pe.Node(interface=mrtrix.FSL2MRTrix(), name='fsl2mrtrix')
    """
    Distortions induced by eddy currents are corrected prior to fitting the tensors.
    The first image is used as a reference for which to warp the others.
    """

    eddycorrect = create_eddy_correct_pipeline(name='eddycorrect')
    eddycorrect.inputs.inputnode.ref_num = 1
    """
    Tensors are fitted to each voxel in the diffusion-weighted image and from these three maps are created:
        * Major eigenvector in each voxel
        * Apparent diffusion coefficient
        * Fractional anisotropy
    """

    dwi2tensor = pe.Node(interface=mrtrix.DWI2Tensor(), name='dwi2tensor')
    tensor2vector = pe.Node(
        interface=mrtrix.Tensor2Vector(), name='tensor2vector')
    tensor2adc = pe.Node(
        interface=mrtrix.Tensor2ApparentDiffusion(), name='tensor2adc')
    tensor2fa = pe.Node(
        interface=mrtrix.Tensor2FractionalAnisotropy(), name='tensor2fa')
    MRconvert_fa = pe.Node(interface=mrtrix.MRConvert(), name='MRconvert_fa')
    MRconvert_fa.inputs.extension = 'nii'
    """

    These nodes are used to create a rough brain mask from the b0 image.
    The b0 image is extracted from the original diffusion-weighted image,
    put through a simple thresholding routine, and smoothed using a 3x3 median filter.
    """

    MRconvert = pe.Node(interface=mrtrix.MRConvert(), name='MRconvert')
    MRconvert.inputs.extract_at_axis = 3
    MRconvert.inputs.extract_at_coordinate = [0]
    threshold_b0 = pe.Node(interface=mrtrix.Threshold(), name='threshold_b0')
    median3d = pe.Node(interface=mrtrix.MedianFilter3D(), name='median3d')
    """
    The brain mask is also used to help identify single-fiber voxels.
    This is done by passing the brain mask through two erosion steps,
    multiplying the remaining mask with the fractional anisotropy map, and
    thresholding the result to obtain some highly anisotropic within-brain voxels.
    """

    erode_mask_firstpass = pe.Node(
        interface=mrtrix.Erode(), name='erode_mask_firstpass')
    erode_mask_secondpass = pe.Node(
        interface=mrtrix.Erode(), name='erode_mask_secondpass')
    MRmultiply = pe.Node(interface=mrtrix.MRMultiply(), name='MRmultiply')
    MRmult_merge = pe.Node(interface=util.Merge(2), name='MRmultiply_merge')
    threshold_FA = pe.Node(interface=mrtrix.Threshold(), name='threshold_FA')
    threshold_FA.inputs.absolute_threshold_value = 0.7
    """
    For whole-brain tracking we also require a broad white-matter seed mask.
    This is created by generating a white matter mask, given a brainmask, and
    thresholding it at a reasonably high level.
    """

    bet = pe.Node(interface=fsl.BET(mask=True), name='bet_b0')
    gen_WM_mask = pe.Node(
        interface=mrtrix.GenerateWhiteMatterMask(), name='gen_WM_mask')
    threshold_wmmask = pe.Node(
        interface=mrtrix.Threshold(), name='threshold_wmmask')
    threshold_wmmask.inputs.absolute_threshold_value = 0.4
    """
    The spherical deconvolution step depends on the estimate of the response function
    in the highly anisotropic voxels we obtained above.

    .. warning::

        For damaged or pathological brains one should take care to lower the maximum harmonic order of these steps.

    """

    estimateresponse = pe.Node(
        interface=mrtrix.EstimateResponseForSH(), name='estimateresponse')
    estimateresponse.inputs.maximum_harmonic_order = 6
    csdeconv = pe.Node(
        interface=mrtrix.ConstrainedSphericalDeconvolution(), name='csdeconv')
    csdeconv.inputs.maximum_harmonic_order = 6
    """
    Finally, we track probabilistically using the orientation distribution functions obtained earlier.
    The tracts are then used to generate a tract-density image, and they are also converted to TrackVis format.
    """

    probCSDstreamtrack = pe.Node(
        interface=mrtrix.ProbabilisticSphericallyDeconvolutedStreamlineTrack(),
        name='probCSDstreamtrack')
    probCSDstreamtrack.inputs.inputmodel = 'SD_PROB'
    probCSDstreamtrack.inputs.desired_number_of_tracks = 150000
    tracks2prob = pe.Node(interface=mrtrix.Tracks2Prob(), name='tracks2prob')
    tracks2prob.inputs.colour = True
    MRconvert_tracks2prob = MRconvert_fa.clone(name='MRconvert_tracks2prob')
    tck2trk = pe.Node(interface=mrtrix.MRTrix2TrackVis(), name='tck2trk')
    trk2tdi = pe.Node(interface=dipy.TrackDensityMap(), name='trk2tdi')
    """
    Structural segmentation nodes
    -----------------------------
    """
    """
    The following node identifies the transformation between the diffusion-weighted
    image and the structural image. This transformation is then applied to the tracts
    so that they are in the same space as the regions of interest.
    """

    coregister = pe.Node(interface=fsl.FLIRT(dof=6), name='coregister')
    coregister.inputs.cost = ('normmi')
    """
    Parcellation is performed given the aparc+aseg image from Freesurfer.
    The CMTK Parcellation step subdivides these regions to return a higher-resolution parcellation scheme.
    The parcellation used here is entitled "scale500" and returns 1015 regions.
    """

    parcellate = pe.Node(interface=cmtk.Parcellate(), name="Parcellate")
    parcellate.inputs.parcellation_name = parcellation_name
    """
    The CreateMatrix interface takes in the remapped aparc+aseg image as well as the label dictionary and fiber tracts
    and outputs a number of different files. The most important of which is the connectivity network itself, which is stored
    as a 'gpickle' and can be loaded using Python's NetworkX package (see CreateMatrix docstring). Also outputted are various
    NumPy arrays containing detailed tract information, such as the start and endpoint regions, and statistics on the mean and
    standard deviation for the fiber length of each connection. These matrices can be used in the ConnectomeViewer to plot the
    specific tracts that connect between user-selected regions.

    Here we choose the Lausanne2008 parcellation scheme, since we are incorporating the CMTK parcellation step.
    """

    creatematrix = pe.Node(interface=cmtk.CreateMatrix(), name="CreateMatrix")
    creatematrix.inputs.count_region_intersections = True
    """
    Next we define the endpoint of this tutorial, which is the CFFConverter node, as well as a few nodes which use
    the Nipype Merge utility. These are useful for passing lists of the files we want packaged in our CFF file.
    The inspect.getfile command is used to package this script into the resulting CFF file, so that it is easy to
    look back at the processing parameters that were used.
    """

    CFFConverter = pe.Node(interface=cmtk.CFFConverter(), name="CFFConverter")
    CFFConverter.inputs.script_files = op.abspath(
        inspect.getfile(inspect.currentframe()))
    giftiSurfaces = pe.Node(interface=util.Merge(8), name="GiftiSurfaces")
    giftiLabels = pe.Node(interface=util.Merge(2), name="GiftiLabels")
    niftiVolumes = pe.Node(interface=util.Merge(3), name="NiftiVolumes")
    fiberDataArrays = pe.Node(interface=util.Merge(4), name="FiberDataArrays")
    """
    We also create a node to calculate several network metrics on our resulting file, and another CFF converter
    which will be used to package these networks into a single file.
    """

    networkx = create_networkx_pipeline(name='networkx')
    cmats_to_csv = create_cmats_to_csv_pipeline(name='cmats_to_csv')
    nfibs_to_csv = pe.Node(interface=misc.Matlab2CSV(), name='nfibs_to_csv')
    merge_nfib_csvs = pe.Node(
        interface=misc.MergeCSVFiles(), name='merge_nfib_csvs')
    merge_nfib_csvs.inputs.extra_column_heading = 'Subject'
    merge_nfib_csvs.inputs.out_file = 'fibers.csv'
    NxStatsCFFConverter = pe.Node(
        interface=cmtk.CFFConverter(), name="NxStatsCFFConverter")
    NxStatsCFFConverter.inputs.script_files = op.abspath(
        inspect.getfile(inspect.currentframe()))
    """
    Connecting the workflow
    =======================
    Here we connect our processing pipeline.
    """
    """
    Connecting the inputs, FreeSurfer nodes, and conversions
    --------------------------------------------------------
    """

    mapping = pe.Workflow(name='mapping')
    """
    First, we connect the input node to the FreeSurfer input nodes.
    """

    mapping.connect([(inputnode_within, FreeSurferSource, [("subjects_dir",
                                                            "subjects_dir")])])
    mapping.connect([(inputnode_within, FreeSurferSource, [("subject_id",
                                                            "subject_id")])])

    mapping.connect([(inputnode_within, FreeSurferSourceLH,
                      [("subjects_dir", "subjects_dir")])])
    mapping.connect([(inputnode_within, FreeSurferSourceLH, [("subject_id",
                                                              "subject_id")])])

    mapping.connect([(inputnode_within, FreeSurferSourceRH,
                      [("subjects_dir", "subjects_dir")])])
    mapping.connect([(inputnode_within, FreeSurferSourceRH, [("subject_id",
                                                              "subject_id")])])

    mapping.connect([(inputnode_within, parcellate, [("subjects_dir",
                                                      "subjects_dir")])])
    mapping.connect([(inputnode_within, parcellate, [("subject_id",
                                                      "subject_id")])])
    mapping.connect([(parcellate, mri_convert_ROI_scale500, [('roi_file',
                                                              'in_file')])])
    """
    Nifti conversion for subject's stripped brain image from Freesurfer:
    """

    mapping.connect([(FreeSurferSource, mri_convert_Brain, [('brain',
                                                             'in_file')])])
    """
    Surface conversions to GIFTI (pial, white, inflated, and sphere for both hemispheres)
    """

    mapping.connect([(FreeSurferSourceLH, mris_convertLH, [('pial',
                                                            'in_file')])])
    mapping.connect([(FreeSurferSourceRH, mris_convertRH, [('pial',
                                                            'in_file')])])
    mapping.connect([(FreeSurferSourceLH, mris_convertLHwhite, [('white',
                                                                 'in_file')])])
    mapping.connect([(FreeSurferSourceRH, mris_convertRHwhite, [('white',
                                                                 'in_file')])])
    mapping.connect([(FreeSurferSourceLH, mris_convertLHinflated,
                      [('inflated', 'in_file')])])
    mapping.connect([(FreeSurferSourceRH, mris_convertRHinflated,
                      [('inflated', 'in_file')])])
    mapping.connect([(FreeSurferSourceLH, mris_convertLHsphere,
                      [('sphere', 'in_file')])])
    mapping.connect([(FreeSurferSourceRH, mris_convertRHsphere,
                      [('sphere', 'in_file')])])
    """
    The annotation files are converted using the pial surface as a map via the MRIsConvert interface.
    One of the functions defined earlier is used to select the lh.aparc.annot and rh.aparc.annot files
    specifically (rather than e.g. rh.aparc.a2009s.annot) from the output list given by the FreeSurferSource.
    """

    mapping.connect([(FreeSurferSourceLH, mris_convertLHlabels,
                      [('pial', 'in_file')])])
    mapping.connect([(FreeSurferSourceRH, mris_convertRHlabels,
                      [('pial', 'in_file')])])
    mapping.connect([(FreeSurferSourceLH, mris_convertLHlabels,
                      [(('annot', select_aparc_annot), 'annot_file')])])
    mapping.connect([(FreeSurferSourceRH, mris_convertRHlabels,
                      [(('annot', select_aparc_annot), 'annot_file')])])
    """
    Diffusion Processing
    --------------------
    Now we connect the tensor computations:
    """

    mapping.connect([(inputnode_within, fsl2mrtrix, [("bvecs", "bvec_file"),
                                                     ("bvals", "bval_file")])])
    mapping.connect([(inputnode_within, eddycorrect, [("dwi",
                                                       "inputnode.in_file")])])
    mapping.connect([(eddycorrect, dwi2tensor, [("outputnode.eddy_corrected",
                                                 "in_file")])])
    mapping.connect([(fsl2mrtrix, dwi2tensor, [("encoding_file",
                                                "encoding_file")])])

    mapping.connect([
        (dwi2tensor, tensor2vector, [['tensor', 'in_file']]),
        (dwi2tensor, tensor2adc, [['tensor', 'in_file']]),
        (dwi2tensor, tensor2fa, [['tensor', 'in_file']]),
    ])
    mapping.connect([(tensor2fa, MRmult_merge, [("FA", "in1")])])
    mapping.connect([(tensor2fa, MRconvert_fa, [("FA", "in_file")])])
    """

    This block creates the rough brain mask to be multiplied, mulitplies it with the
    fractional anisotropy image, and thresholds it to get the single-fiber voxels.
    """

    mapping.connect([(eddycorrect, MRconvert, [("outputnode.eddy_corrected",
                                                "in_file")])])
    mapping.connect([(MRconvert, threshold_b0, [("converted", "in_file")])])
    mapping.connect([(threshold_b0, median3d, [("out_file", "in_file")])])
    mapping.connect([(median3d, erode_mask_firstpass, [("out_file",
                                                        "in_file")])])
    mapping.connect([(erode_mask_firstpass, erode_mask_secondpass,
                      [("out_file", "in_file")])])
    mapping.connect([(erode_mask_secondpass, MRmult_merge, [("out_file",
                                                             "in2")])])
    mapping.connect([(MRmult_merge, MRmultiply, [("out", "in_files")])])
    mapping.connect([(MRmultiply, threshold_FA, [("out_file", "in_file")])])
    """
    Here the thresholded white matter mask is created for seeding the tractography.
    """

    mapping.connect([(eddycorrect, bet, [("outputnode.eddy_corrected",
                                          "in_file")])])
    mapping.connect([(eddycorrect, gen_WM_mask, [("outputnode.eddy_corrected",
                                                  "in_file")])])
    mapping.connect([(bet, gen_WM_mask, [("mask_file", "binary_mask")])])
    mapping.connect([(fsl2mrtrix, gen_WM_mask, [("encoding_file",
                                                 "encoding_file")])])
    mapping.connect([(gen_WM_mask, threshold_wmmask, [("WMprobabilitymap",
                                                       "in_file")])])
    """
    Next we estimate the fiber response distribution.
    """

    mapping.connect([(eddycorrect, estimateresponse,
                      [("outputnode.eddy_corrected", "in_file")])])
    mapping.connect([(fsl2mrtrix, estimateresponse, [("encoding_file",
                                                      "encoding_file")])])
    mapping.connect([(threshold_FA, estimateresponse, [("out_file",
                                                        "mask_image")])])
    """
    Run constrained spherical deconvolution.
    """

    mapping.connect([(eddycorrect, csdeconv, [("outputnode.eddy_corrected",
                                               "in_file")])])
    mapping.connect([(gen_WM_mask, csdeconv, [("WMprobabilitymap",
                                               "mask_image")])])
    mapping.connect([(estimateresponse, csdeconv, [("response",
                                                    "response_file")])])
    mapping.connect([(fsl2mrtrix, csdeconv, [("encoding_file",
                                              "encoding_file")])])
    """
    Connect the tractography and compute the tract density image.
    """

    mapping.connect([(threshold_wmmask, probCSDstreamtrack, [("out_file",
                                                              "seed_file")])])
    mapping.connect([(csdeconv, probCSDstreamtrack,
                      [("spherical_harmonics_image", "in_file")])])
    mapping.connect([(probCSDstreamtrack, tracks2prob, [("tracked",
                                                         "in_file")])])
    mapping.connect([(eddycorrect, tracks2prob, [("outputnode.eddy_corrected",
                                                  "template_file")])])
    mapping.connect([(tracks2prob, MRconvert_tracks2prob, [("tract_image",
                                                            "in_file")])])
    """
    Structural Processing
    ---------------------
    First, we coregister the diffusion image to the structural image
    """

    mapping.connect([(eddycorrect, coregister, [("outputnode.eddy_corrected",
                                                 "in_file")])])
    mapping.connect([(mri_convert_Brain, coregister, [('out_file',
                                                       'reference')])])
    """
    The MRtrix-tracked fibers are converted to TrackVis format (with voxel and data dimensions grabbed from the DWI).
    The connectivity matrix is created with the transformed .trk fibers and the parcellation file.
    """

    mapping.connect([(eddycorrect, tck2trk, [("outputnode.eddy_corrected",
                                              "image_file")])])
    mapping.connect([(mri_convert_Brain, tck2trk,
                      [("out_file", "registration_image_file")])])
    mapping.connect([(coregister, tck2trk, [("out_matrix_file",
                                             "matrix_file")])])
    mapping.connect([(probCSDstreamtrack, tck2trk, [("tracked", "in_file")])])
    mapping.connect([(tck2trk, creatematrix, [("out_file", "tract_file")])])
    mapping.connect([(tck2trk, trk2tdi, [("out_file", "in_file")])])
    mapping.connect(inputnode_within, 'resolution_network_file', creatematrix,
                    'resolution_network_file')
    mapping.connect([(inputnode_within, creatematrix, [("subject_id",
                                                        "out_matrix_file")])])
    mapping.connect([(inputnode_within, creatematrix,
                      [("subject_id", "out_matrix_mat_file")])])
    mapping.connect([(parcellate, creatematrix, [("roi_file", "roi_file")])])
    """
    The merge nodes defined earlier are used here to create lists of the files which are
    destined for the CFFConverter.
    """

    mapping.connect([(mris_convertLH, giftiSurfaces, [("converted", "in1")])])
    mapping.connect([(mris_convertRH, giftiSurfaces, [("converted", "in2")])])
    mapping.connect([(mris_convertLHwhite, giftiSurfaces, [("converted",
                                                            "in3")])])
    mapping.connect([(mris_convertRHwhite, giftiSurfaces, [("converted",
                                                            "in4")])])
    mapping.connect([(mris_convertLHinflated, giftiSurfaces, [("converted",
                                                               "in5")])])
    mapping.connect([(mris_convertRHinflated, giftiSurfaces, [("converted",
                                                               "in6")])])
    mapping.connect([(mris_convertLHsphere, giftiSurfaces, [("converted",
                                                             "in7")])])
    mapping.connect([(mris_convertRHsphere, giftiSurfaces, [("converted",
                                                             "in8")])])

    mapping.connect([(mris_convertLHlabels, giftiLabels, [("converted",
                                                           "in1")])])
    mapping.connect([(mris_convertRHlabels, giftiLabels, [("converted",
                                                           "in2")])])

    mapping.connect([(parcellate, niftiVolumes, [("roi_file", "in1")])])
    mapping.connect([(eddycorrect, niftiVolumes, [("outputnode.eddy_corrected",
                                                   "in2")])])
    mapping.connect([(mri_convert_Brain, niftiVolumes, [("out_file", "in3")])])

    mapping.connect([(creatematrix, fiberDataArrays, [("endpoint_file",
                                                       "in1")])])
    mapping.connect([(creatematrix, fiberDataArrays, [("endpoint_file_mm",
                                                       "in2")])])
    mapping.connect([(creatematrix, fiberDataArrays, [("fiber_length_file",
                                                       "in3")])])
    mapping.connect([(creatematrix, fiberDataArrays, [("fiber_label_file",
                                                       "in4")])])
    """
    This block actually connects the merged lists to the CFF converter. We pass the surfaces
    and volumes that are to be included, as well as the tracts and the network itself. The currently
    running pipeline (dmri_connectivity_advanced.py) is also scraped and included in the CFF file. This
    makes it easy for the user to examine the entire processing pathway used to generate the end
    product.
    """

    mapping.connect([(giftiSurfaces, CFFConverter, [("out",
                                                     "gifti_surfaces")])])
    mapping.connect([(giftiLabels, CFFConverter, [("out", "gifti_labels")])])
    mapping.connect([(creatematrix, CFFConverter, [("matrix_files",
                                                    "gpickled_networks")])])
    mapping.connect([(niftiVolumes, CFFConverter, [("out", "nifti_volumes")])])
    mapping.connect([(fiberDataArrays, CFFConverter, [("out", "data_files")])])
    mapping.connect([(creatematrix, CFFConverter, [("filtered_tractography",
                                                    "tract_files")])])
    mapping.connect([(inputnode_within, CFFConverter, [("subject_id",
                                                        "title")])])
    """
    The graph theoretical metrics which have been generated are placed into another CFF file.
    """

    mapping.connect([(inputnode_within, networkx,
                      [("subject_id", "inputnode.extra_field")])])
    mapping.connect([(creatematrix, networkx, [("intersection_matrix_file",
                                                "inputnode.network_file")])])

    mapping.connect([(networkx, NxStatsCFFConverter,
                      [("outputnode.network_files", "gpickled_networks")])])
    mapping.connect([(giftiSurfaces, NxStatsCFFConverter,
                      [("out", "gifti_surfaces")])])
    mapping.connect([(giftiLabels, NxStatsCFFConverter, [("out",
                                                          "gifti_labels")])])
    mapping.connect([(niftiVolumes, NxStatsCFFConverter, [("out",
                                                           "nifti_volumes")])])
    mapping.connect([(fiberDataArrays, NxStatsCFFConverter, [("out",
                                                              "data_files")])])
    mapping.connect([(inputnode_within, NxStatsCFFConverter, [("subject_id",
                                                               "title")])])

    mapping.connect([(inputnode_within, cmats_to_csv,
                      [("subject_id", "inputnode.extra_field")])])
    mapping.connect([(creatematrix, cmats_to_csv,
                      [("matlab_matrix_files",
                        "inputnode.matlab_matrix_files")])])
    mapping.connect([(creatematrix, nfibs_to_csv, [("stats_file",
                                                    "in_file")])])
    mapping.connect([(nfibs_to_csv, merge_nfib_csvs, [("csv_files",
                                                       "in_files")])])
    mapping.connect([(inputnode_within, merge_nfib_csvs, [("subject_id",
                                                           "extra_field")])])
    """
    Create a higher-level workflow
    --------------------------------------
    Finally, we create another higher-level workflow to connect our mapping workflow with the info and datagrabbing nodes
    declared at the beginning. Our tutorial can is now extensible to any arbitrary number of subjects by simply adding
    their names to the subject list and their data to the proper folders.
    """

    inputnode = pe.Node(
        interface=util.IdentityInterface(
            fields=["subject_id", "dwi", "bvecs", "bvals", "subjects_dir"]),
        name="inputnode")

    outputnode = pe.Node(
        interface=util.IdentityInterface(fields=[
            "fa", "struct", "tracts", "tracks2prob", "connectome",
            "nxstatscff", "nxmatlab", "nxcsv", "fiber_csv", "cmatrices_csv",
            "nxmergedcsv", "cmatrix", "networks", "filtered_tracts", "rois",
            "odfs", "tdi", "mean_fiber_length", "median_fiber_length",
            "fiber_length_std"
        ]),
        name="outputnode")

    connectivity = pe.Workflow(name="connectivity")
    connectivity.base_output_dir = name
    connectivity.base_dir = name

    connectivity.connect([
        (inputnode, mapping,
         [("dwi", "inputnode_within.dwi"), ("bvals", "inputnode_within.bvals"),
          ("bvecs", "inputnode_within.bvecs"), ("subject_id",
                                                "inputnode_within.subject_id"),
          ("subjects_dir", "inputnode_within.subjects_dir")])
    ])

    connectivity.connect(
        [(mapping, outputnode,
          [("tck2trk.out_file",
            "tracts"), ("CFFConverter.connectome_file",
                        "connectome"), ("NxStatsCFFConverter.connectome_file",
                                        "nxstatscff"),
           ("CreateMatrix.matrix_mat_file",
            "cmatrix"), ("CreateMatrix.mean_fiber_length_matrix_mat_file",
                         "mean_fiber_length"),
           ("CreateMatrix.median_fiber_length_matrix_mat_file",
            "median_fiber_length"),
           ("CreateMatrix.fiber_length_std_matrix_mat_file",
            "fiber_length_std"), ("CreateMatrix.matrix_files", "networks"),
           ("CreateMatrix.filtered_tractographies",
            "filtered_tracts"), ("merge_nfib_csvs.csv_file", "fiber_csv"),
           ("mri_convert_ROI_scale500.out_file",
            "rois"), ("trk2tdi.out_file",
                      "tdi"), ("csdeconv.spherical_harmonics_image", "odfs"),
           ("mri_convert_Brain.out_file",
            "struct"), ("MRconvert_fa.converted",
                        "fa"), ("MRconvert_tracks2prob.converted",
                                "tracks2prob")])])

    connectivity.connect([(cmats_to_csv, outputnode, [("outputnode.csv_file",
                                                       "cmatrices_csv")])])
    connectivity.connect([(networkx, outputnode, [("outputnode.csv_files",
                                                   "nxcsv")])])
    return connectivity



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from . import ants
from . import freesurfer
from . import niftyreg



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from builtins import range
from ....pipeline import engine as pe
from ....interfaces import utility as niu
from ....interfaces import dipy


def nlmeans_pipeline(name='Denoise',
                     params={
                         'patch_radius': 1,
                         'block_radius': 5
                     }):
    """
    Workflow that performs nlmeans denoising

    Example
    -------

    >>> from nipype.workflows.dmri.dipy.denoise import nlmeans_pipeline
    >>> denoise = nlmeans_pipeline()
    >>> denoise.inputs.inputnode.in_file = 'diffusion.nii'
    >>> denoise.inputs.inputnode.in_mask = 'mask.nii'
    >>> denoise.run() # doctest: +SKIP


    """
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['in_file', 'in_mask']), name='inputnode')
    outputnode = pe.Node(
        niu.IdentityInterface(fields=['out_file']), name='outputnode')

    nmask = pe.Node(
        niu.Function(
            input_names=['in_file', 'in_mask'],
            output_names=['out_file'],
            function=bg_mask),
        name='NoiseMsk')
    nlmeans = pe.Node(dipy.Denoise(**params), name='NLMeans')

    wf = pe.Workflow(name=name)
    wf.connect([(inputnode, nmask, [
        ('in_file', 'in_file'), ('in_mask', 'in_mask')
    ]), (inputnode, nlmeans, [('in_file', 'in_file'), ('in_mask', 'in_mask')]),
                (nmask, nlmeans, [('out_file', 'noise_mask')]),
                (nlmeans, outputnode, [('out_file', 'out_file')])])
    return wf


def csf_mask(in_file, in_mask, out_file=None):
    """
    Artesanal mask of csf in T2w-like images
    """
    import nibabel as nb
    import numpy as np
    from scipy.ndimage import binary_erosion, binary_opening, label
    import scipy.ndimage as nd
    import os.path as op
    from nipype.utils import NUMPY_MMAP

    if out_file is None:
        fname, ext = op.splitext(op.basename(in_file))
        if ext == ".gz":
            fname, ext2 = op.splitext(fname)
            ext = ext2 + ext
        out_file = op.abspath("%s_csfmask%s" % (fname, ext))

    im = nb.load(in_file, mmap=NUMPY_MMAP)
    hdr = im.header.copy()
    hdr.set_data_dtype(np.uint8)
    hdr.set_xyzt_units('mm')
    imdata = im.get_data()
    msk = nb.load(in_mask, mmap=NUMPY_MMAP).get_data()
    msk = binary_erosion(msk, structure=np.ones((15, 15, 10))).astype(np.uint8)
    thres = np.percentile(imdata[msk > 0].reshape(-1), 90.0)
    imdata[imdata < thres] = 0
    imdata = imdata * msk
    imdata[imdata > 0] = 1
    imdata = binary_opening(
        imdata, structure=np.ones((2, 2, 2))).astype(np.uint8)

    label_im, nb_labels = label(imdata)
    sizes = nd.sum(imdata, label_im, list(range(nb_labels + 1)))
    mask_size = sizes != sizes.max()
    remove_pixel = mask_size[label_im]
    label_im[remove_pixel] = 0
    label_im[label_im > 0] = 1
    nb.Nifti1Image(label_im.astype(np.uint8), im.affine,
                   hdr).to_filename(out_file)
    return out_file


def bg_mask(in_file, in_mask, out_file=None):
    """
    Rough mask of background from brain masks
    """
    import nibabel as nb
    import numpy as np
    from scipy.ndimage import binary_dilation
    import scipy.ndimage as nd
    import os.path as op
    from nipype.utils import NUMPY_MMAP

    if out_file is None:
        fname, ext = op.splitext(op.basename(in_file))
        if ext == ".gz":
            fname, ext2 = op.splitext(fname)
            ext = ext2 + ext
        out_file = op.abspath("%s_bgmask%s" % (fname, ext))

    im = nb.load(in_file, mmap=NUMPY_MMAP)
    hdr = im.header.copy()
    hdr.set_data_dtype(np.uint8)
    hdr.set_xyzt_units('mm')
    msk = nb.load(in_mask, mmap=NUMPY_MMAP).get_data()
    msk = 1 - binary_dilation(msk, structure=np.ones((20, 20, 20)))
    nb.Nifti1Image(msk.astype(np.uint8), im.affine, hdr).to_filename(out_file)
    return out_file



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from __future__ import absolute_import
from .tensor_registration import (affine_tensor_pipeline,
                                  diffeomorphic_tensor_pipeline)



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from ....pipeline import engine as pe
from ....interfaces import utility as niu
from ....interfaces import dtitk


def affine_tensor_pipeline(name='AffTen'):

    """
    Workflow that performs a linear registration
    (Rigid followed by Affine)

    Example
    -------

    >>> from nipype.workflows.dmri.dtitk.tensor_registration import affine_tensor_pipeline
    >>> affine = affine_tensor_pipeline()
    >>> affine.inputs.inputnode.fixed_file = 'im1.nii'
    >>> affine.inputs.inputnode.moving_file = 'im2.nii'
    >>> affine.run() # doctest: +SKIP


    """
    inputnode = pe.Node(niu.IdentityInterface(
                        fields=['fixed_file', 'moving_file']),
                        name='inputnode')
    outputnode = pe.Node(niu.IdentityInterface(
                         fields=['out_file', 'out_file_xfm']),
                         name='outputnode')

    rigid_node = pe.Node(dtitk.Rigid(), name='rigid_node')
    affine_node = pe.Node(dtitk.Affine(), name='affine_node')

    wf = pe.Workflow(name=name)

    wf.connect(inputnode, 'fixed_file', rigid_node, 'fixed_file')
    wf.connect(inputnode, 'moving_file', rigid_node, 'moving_file')
    wf.connect(rigid_node, 'out_file_xfm', affine_node, 'initialize_xfm')
    wf.connect(inputnode, 'fixed_file', affine_node, 'fixed_file')
    wf.connect(inputnode, 'moving_file', affine_node, 'moving_file')
    wf.connect(affine_node, 'out_file', outputnode, 'out_file')
    wf.connect(affine_node, 'out_file_xfm', outputnode, 'out_file_xfm')

    return wf


def diffeomorphic_tensor_pipeline(name='DiffeoTen',
                                  params={'array_size': (128, 128, 64)}):
    """
    Workflow that performs a diffeomorphic registration
    (Rigid and Affine follwed by Diffeomorphic)
    Note: the requirements for a diffeomorphic registration specify that
    the dimension 0 is a power of 2 so images are resliced prior to
    registration

    Example
    -------

    >>> from nipype.workflows.dmri.dtitk.tensor_registration import diffeomorphic_tensor_pipeline
    >>> diffeo = diffeomorphic_tensor_pipeline()
    >>> diffeo.inputs.inputnode.fixed_file = 'im1.nii'
    >>> diffeo.inputs.inputnode.moving_file = 'im2.nii'
    >>> diffeo.run() # doctest: +SKIP


    """
    inputnode = pe.Node(niu.IdentityInterface(
                        fields=['fixed_file', 'moving_file']),
                        name='inputnode')
    outputnode = pe.Node(niu.IdentityInterface(
                         fields=['out_file', 'out_file_xfm',
                                 'fixed_resliced', 'moving_resliced']),
                         name='outputnode')

    reslice_node_pow2 = pe.Node(dtitk.TVResample(
                                origin=(0, 0, 0),
                                array_size=params['array_size']),
                                name='reslice_node_pow2')
    reslice_node_moving = pe.Node(dtitk.TVResample(),
                                  name='reslice_node_moving')
    mask_node = pe.Node(dtitk.BinThresh(lower_bound=0.01, upper_bound=100,
                                        inside_value=1, outside_value=0),
                        name='mask_node')
    rigid_node = pe.Node(dtitk.Rigid(), name='rigid_node')
    affine_node = pe.Node(dtitk.Affine(), name='affine_node')
    diffeo_node = pe.Node(dtitk.Diffeo(n_iters=6, ftol=0.002),
                          name='diffeo_node')
    compose_xfm_node = pe.Node(dtitk.ComposeXfm(), name='compose_xfm_node')
    apply_xfm_node = pe.Node(dtitk.DiffeoSymTensor3DVol(),
                             name='apply_xfm_node')

    wf = pe.Workflow(name=name)

    # Reslice input images
    wf.connect(inputnode, 'fixed_file', reslice_node_pow2, 'in_file')
    wf.connect(reslice_node_pow2, 'out_file',
               reslice_node_moving, 'target_file')
    wf.connect(inputnode, 'moving_file', reslice_node_moving, 'in_file')
    # Rigid registration
    wf.connect(reslice_node_pow2, 'out_file', rigid_node, 'fixed_file')
    wf.connect(reslice_node_moving, 'out_file', rigid_node, 'moving_file')
    # Affine registration
    wf.connect(rigid_node, 'out_file_xfm', affine_node, 'initialize_xfm')
    wf.connect(reslice_node_pow2, 'out_file', affine_node, 'fixed_file')
    wf.connect(reslice_node_moving, 'out_file', affine_node, 'moving_file')
    # Diffeo registration
    wf.connect(reslice_node_pow2, 'out_file', mask_node, 'in_file')
    wf.connect(reslice_node_pow2, 'out_file', diffeo_node, 'fixed_file')
    wf.connect(affine_node, 'out_file', diffeo_node, 'moving_file')
    wf.connect(mask_node, 'out_file', diffeo_node, 'mask_file')
    # Compose transform
    wf.connect(diffeo_node, 'out_file_xfm', compose_xfm_node, 'in_df')
    wf.connect(affine_node, 'out_file_xfm', compose_xfm_node, 'in_aff')
    # Apply transform
    wf.connect(reslice_node_moving, 'out_file', apply_xfm_node, 'in_file')
    wf.connect(compose_xfm_node, 'out_file', apply_xfm_node, 'transform')
    # Send to output
    wf.connect(apply_xfm_node, 'out_file', outputnode, 'out_file')
    wf.connect(compose_xfm_node, 'out_file', outputnode, 'out_file_xfm')
    wf.connect(reslice_node_pow2, 'out_file', outputnode, 'fixed_resliced')
    wf.connect(reslice_node_moving, 'out_file', outputnode, 'moving_resliced')

    return wf



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ImageMaths


def test_ImageMaths_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        ignore_exception=dict(
            deprecated='1.0.0',
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            mandatory=True,
            position=1,
        ),
        in_file2=dict(
            argstr='%s',
            position=3,
        ),
        mask_file=dict(argstr='-mas %s', ),
        op_string=dict(
            argstr='%s',
            position=2,
        ),
        out_data_type=dict(
            argstr='-odt %s',
            position=-1,
        ),
        out_file=dict(
            argstr='%s',
            genfile=True,
            hash_files=False,
            position=-2,
        ),
        output_type=dict(),
        suffix=dict(),
        terminal_output=dict(
            deprecated='1.0.0',
            nohash=True,
        ),
    )
    inputs = ImageMaths.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ImageMaths_outputs():
    output_map = dict(out_file=dict(), )
    outputs = ImageMaths.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ParcellationStats


def test_ParcellationStats_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        aseg=dict(mandatory=True, ),
        brainmask=dict(mandatory=True, ),
        copy_inputs=dict(),
        cortex_label=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hemisphere=dict(
            argstr='%s',
            mandatory=True,
            position=-2,
        ),
        ignore_exception=dict(
            deprecated='1.0.0',
            nohash=True,
            usedefault=True,
        ),
        in_annotation=dict(
            argstr='-a %s',
            xor=['in_label'],
        ),
        in_cortex=dict(argstr='-cortex %s', ),
        in_label=dict(
            argstr='-l %s',
            xor=['in_annotatoin', 'out_color'],
        ),
        lh_pial=dict(mandatory=True, ),
        lh_white=dict(mandatory=True, ),
        mgz=dict(argstr='-mgz', ),
        out_color=dict(
            argstr='-c %s',
            genfile=True,
            xor=['in_label'],
        ),
        out_table=dict(
            argstr='-f %s',
            genfile=True,
            requires=['tabular_output'],
        ),
        rh_pial=dict(mandatory=True, ),
        rh_white=dict(mandatory=True, ),
        ribbon=dict(mandatory=True, ),
        subject_id=dict(
            argstr='%s',
            mandatory=True,
            position=-3,
            usedefault=True,
        ),
        subjects_dir=dict(),
        surface=dict(
            argstr='%s',
            position=-1,
        ),
        tabular_output=dict(argstr='-b', ),
        terminal_output=dict(
            deprecated='1.0.0',
            nohash=True,
        ),
        th3=dict(
            argstr='-th3',
            requires=['cortex_label'],
        ),
        thickness=dict(mandatory=True, ),
        transform=dict(mandatory=True, ),
        wm=dict(mandatory=True, ),
    )
    inputs = ParcellationStats.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ParcellationStats_outputs():
    output_map = dict(
        out_color=dict(),
        out_table=dict(),
    )
    outputs = ParcellationStats.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import unicode_literals
from builtins import open

import os
import time
import warnings

import pytest
from ...testing import TempFATFS
from ...utils.filemanip import (
    save_json, load_json, fname_presuffix, fnames_presuffix, hash_rename,
    check_forhash, _parse_mount_table, _cifs_table, on_cifs, copyfile,
    copyfiles, ensure_list, simplify_list, check_depends,
    split_filename, get_related_files, indirectory)


def _ignore_atime(stat):
    return stat[:7] + stat[8:]


@pytest.mark.parametrize(
    "filename, split",
    [('foo.nii', ('', 'foo', '.nii')), ('foo.nii.gz', ('', 'foo', '.nii.gz')),
     ('foo.niml.dset', ('', 'foo', '.niml.dset')),
     ('/usr/local/foo.nii.gz',
      ('/usr/local', 'foo', '.nii.gz')), ('../usr/local/foo.nii',
                                          ('../usr/local', 'foo', '.nii')),
     ('/usr/local/foo.a.b.c.d',
      ('/usr/local', 'foo.a.b.c', '.d')), ('/usr/local/',
                                           ('/usr/local', '', ''))])
def test_split_filename(filename, split):
    res = split_filename(filename)
    assert res == split


def test_fname_presuffix():
    fname = 'foo.nii'
    pth = fname_presuffix(fname, 'pre_', '_post', '/tmp')
    assert pth == '/tmp/pre_foo_post.nii'
    fname += '.gz'
    pth = fname_presuffix(fname, 'pre_', '_post', '/tmp')
    assert pth == '/tmp/pre_foo_post.nii.gz'
    pth = fname_presuffix(fname, 'pre_', '_post', '/tmp', use_ext=False)
    assert pth == '/tmp/pre_foo_post'


def test_fnames_presuffix():
    fnames = ['foo.nii', 'bar.nii']
    pths = fnames_presuffix(fnames, 'pre_', '_post', '/tmp')
    assert pths == ['/tmp/pre_foo_post.nii', '/tmp/pre_bar_post.nii']


@pytest.mark.parametrize("filename, newname",
                         [('foobar.nii', 'foobar_0xabc123.nii'),
                          ('foobar.nii.gz', 'foobar_0xabc123.nii.gz')])
def test_hash_rename(filename, newname):
    new_name = hash_rename(filename, 'abc123')
    assert new_name == newname


def test_check_forhash():
    fname = 'foobar'
    orig_hash = '_0x4323dbcefdc51906decd8edcb3327943'
    hashed_name = ''.join((fname, orig_hash, '.nii'))
    result, hash = check_forhash(hashed_name)
    assert result
    assert hash == [orig_hash]
    result, hash = check_forhash('foobar.nii')
    assert not result
    assert hash is None


@pytest.fixture()
def _temp_analyze_files(tmpdir):
    """Generate temporary analyze file pair."""
    orig_img = tmpdir.join("orig.img")
    orig_hdr = tmpdir.join("orig.hdr")
    orig_img.open('w+').close()
    orig_hdr.open('w+').close()
    return str(orig_img), str(orig_hdr)


@pytest.fixture()
def _temp_analyze_files_prime(tmpdir):
    """Generate temporary analyze file pair."""
    orig_img = tmpdir.join("orig_prime.img")
    orig_hdr = tmpdir.join("orig_prime.hdr")
    orig_img.open('w+').close()
    orig_hdr.open('w+').close()
    return orig_img.strpath, orig_hdr.strpath


def test_copyfile(_temp_analyze_files):
    orig_img, orig_hdr = _temp_analyze_files
    pth, fname = os.path.split(orig_img)
    new_img = os.path.join(pth, 'newfile.img')
    new_hdr = os.path.join(pth, 'newfile.hdr')
    copyfile(orig_img, new_img)
    assert os.path.exists(new_img)
    assert os.path.exists(new_hdr)


def test_copyfile_true(_temp_analyze_files):
    orig_img, orig_hdr = _temp_analyze_files
    pth, fname = os.path.split(orig_img)
    new_img = os.path.join(pth, 'newfile.img')
    new_hdr = os.path.join(pth, 'newfile.hdr')
    # Test with copy=True
    copyfile(orig_img, new_img, copy=True)
    assert os.path.exists(new_img)
    assert os.path.exists(new_hdr)


def test_copyfiles(_temp_analyze_files, _temp_analyze_files_prime):
    orig_img1, orig_hdr1 = _temp_analyze_files
    orig_img2, orig_hdr2 = _temp_analyze_files_prime
    pth, fname = os.path.split(orig_img1)
    new_img1 = os.path.join(pth, 'newfile.img')
    new_hdr1 = os.path.join(pth, 'newfile.hdr')
    pth, fname = os.path.split(orig_img2)
    new_img2 = os.path.join(pth, 'secondfile.img')
    new_hdr2 = os.path.join(pth, 'secondfile.hdr')
    copyfiles([orig_img1, orig_img2], [new_img1, new_img2])
    assert os.path.exists(new_img1)
    assert os.path.exists(new_hdr1)
    assert os.path.exists(new_img2)
    assert os.path.exists(new_hdr2)


def test_linkchain(_temp_analyze_files):
    if os.name is not 'posix':
        return
    orig_img, orig_hdr = _temp_analyze_files
    pth, fname = os.path.split(orig_img)
    new_img1 = os.path.join(pth, 'newfile1.img')
    new_hdr1 = os.path.join(pth, 'newfile1.hdr')
    new_img2 = os.path.join(pth, 'newfile2.img')
    new_hdr2 = os.path.join(pth, 'newfile2.hdr')
    new_img3 = os.path.join(pth, 'newfile3.img')
    new_hdr3 = os.path.join(pth, 'newfile3.hdr')
    copyfile(orig_img, new_img1)
    assert os.path.islink(new_img1)
    assert os.path.islink(new_hdr1)
    copyfile(new_img1, new_img2, copy=True)
    assert not os.path.islink(new_img2)
    assert not os.path.islink(new_hdr2)
    assert not os.path.samefile(orig_img, new_img2)
    assert not os.path.samefile(orig_hdr, new_hdr2)
    copyfile(new_img1, new_img3, copy=True, use_hardlink=True)
    assert not os.path.islink(new_img3)
    assert not os.path.islink(new_hdr3)
    assert os.path.samefile(orig_img, new_img3)
    assert os.path.samefile(orig_hdr, new_hdr3)


def test_recopy(_temp_analyze_files):
    # Re-copying with the same parameters on an unchanged file should be
    # idempotent
    #
    # Test for copying from regular files and symlinks
    orig_img, orig_hdr = _temp_analyze_files
    pth, fname = os.path.split(orig_img)
    img_link = os.path.join(pth, 'imglink.img')
    new_img = os.path.join(pth, 'newfile.img')
    new_hdr = os.path.join(pth, 'newfile.hdr')
    copyfile(orig_img, img_link)
    for copy in (True, False):
        for use_hardlink in (True, False):
            for hashmethod in ('timestamp', 'content'):
                kwargs = {
                    'copy': copy,
                    'use_hardlink': use_hardlink,
                    'hashmethod': hashmethod
                }
                # Copying does not preserve the original file's timestamp, so
                # we may delete and re-copy, if the test is slower than a clock
                # tick
                if copy and not use_hardlink and hashmethod == 'timestamp':
                    continue

                copyfile(orig_img, new_img, **kwargs)
                img_stat = _ignore_atime(os.stat(new_img))
                hdr_stat = _ignore_atime(os.stat(new_hdr))
                copyfile(orig_img, new_img, **kwargs)
                err_msg = "Regular - OS: {}; Copy: {}; Hardlink: {}".format(
                    os.name, copy, use_hardlink)
                assert img_stat == _ignore_atime(os.stat(new_img)), err_msg
                assert hdr_stat == _ignore_atime(os.stat(new_hdr)), err_msg
                os.unlink(new_img)
                os.unlink(new_hdr)

                copyfile(img_link, new_img, **kwargs)
                img_stat = _ignore_atime(os.stat(new_img))
                hdr_stat = _ignore_atime(os.stat(new_hdr))
                copyfile(img_link, new_img, **kwargs)
                err_msg = "Symlink - OS: {}; Copy: {}; Hardlink: {}".format(
                    os.name, copy, use_hardlink)
                assert img_stat == _ignore_atime(os.stat(new_img)), err_msg
                assert hdr_stat == _ignore_atime(os.stat(new_hdr)), err_msg
                os.unlink(new_img)
                os.unlink(new_hdr)


def test_copyfallback(_temp_analyze_files):
    if os.name is not 'posix':
        return
    orig_img, orig_hdr = _temp_analyze_files
    pth, imgname = os.path.split(orig_img)
    pth, hdrname = os.path.split(orig_hdr)
    try:
        fatfs = TempFATFS()
    except (IOError, OSError):
        warnings.warn('Fuse mount failed. copyfile fallback tests skipped.')
    else:
        with fatfs as fatdir:
            tgt_img = os.path.join(fatdir, imgname)
            tgt_hdr = os.path.join(fatdir, hdrname)
            for copy in (True, False):
                for use_hardlink in (True, False):
                    copyfile(
                        orig_img,
                        tgt_img,
                        copy=copy,
                        use_hardlink=use_hardlink)
                    assert os.path.exists(tgt_img)
                    assert os.path.exists(tgt_hdr)
                    assert not os.path.islink(tgt_img)
                    assert not os.path.islink(tgt_hdr)
                    assert not os.path.samefile(orig_img, tgt_img)
                    assert not os.path.samefile(orig_hdr, tgt_hdr)
                    os.unlink(tgt_img)
                    os.unlink(tgt_hdr)


def test_get_related_files(_temp_analyze_files):
    orig_img, orig_hdr = _temp_analyze_files

    related_files = get_related_files(orig_img)
    assert orig_img in related_files
    assert orig_hdr in related_files

    related_files = get_related_files(orig_hdr)
    assert orig_img in related_files
    assert orig_hdr in related_files


def test_get_related_files_noninclusive(_temp_analyze_files):
    orig_img, orig_hdr = _temp_analyze_files

    related_files = get_related_files(orig_img, include_this_file=False)
    assert orig_img not in related_files
    assert orig_hdr in related_files

    related_files = get_related_files(orig_hdr, include_this_file=False)
    assert orig_img in related_files
    assert orig_hdr not in related_files


@pytest.mark.parametrize("filename, expected",
                         [('foo.nii', ['foo.nii']), (['foo.nii'], ['foo.nii']),
                          (('foo', 'bar'), ['foo', 'bar']), (12.34, None)])
def test_ensure_list(filename, expected):
    x = ensure_list(filename)
    assert x == expected


@pytest.mark.parametrize("list, expected", [
    (['foo.nii'], 'foo.nii'),
    (['foo', 'bar'], ['foo', 'bar']),
])
def test_simplify_list(list, expected):
    x = simplify_list(list)
    assert x == expected


def test_check_depends(tmpdir):
    def touch(fname):
        with open(fname, 'a'):
            os.utime(fname, None)

    dependencies = [tmpdir.join(str(i)).strpath for i in range(3)]
    targets = [tmpdir.join(str(i)).strpath for i in range(3, 6)]

    # Targets newer than dependencies
    for dep in dependencies:
        touch(dep)
    time.sleep(1)
    for tgt in targets:
        touch(tgt)
    assert check_depends(targets, dependencies)

    # Targets older than newest dependency
    time.sleep(1)
    touch(dependencies[0])
    assert not check_depends(targets, dependencies)

    # Missing dependency
    os.unlink(dependencies[0])
    try:
        check_depends(targets, dependencies)
    except OSError:
        pass
    else:
        assert False, "Should raise OSError on missing dependency"


def test_json(tmpdir):
    # Simple roundtrip test of json files, just a sanity check.
    adict = dict(a='one', c='three', b='two')
    name = tmpdir.join('test.json').strpath
    save_json(name, adict)  # save_json closes the file
    new_dict = load_json(name)
    os.unlink(name)
    assert sorted(adict.items()) == sorted(new_dict.items())


@pytest.mark.parametrize(
    "file, length, expected_files",
    [('/path/test.img', 3,
      ['/path/test.hdr', '/path/test.img', '/path/test.mat']),
     ('/path/test.hdr', 3,
      ['/path/test.hdr', '/path/test.img', '/path/test.mat']),
     ('/path/test.BRIK', 2, ['/path/test.BRIK', '/path/test.HEAD']),
     ('/path/test.HEAD', 2, ['/path/test.BRIK', '/path/test.HEAD']),
     ('/path/foo.nii', 2, ['/path/foo.nii', '/path/foo.mat'])])
def test_related_files(file, length, expected_files):
    related_files = get_related_files(file)

    assert len(related_files) == length

    for ef in expected_files:
        assert ef in related_files


MOUNT_OUTPUTS = (
# Linux, no CIFS
(r'''sysfs on /sys type sysfs (rw,nosuid,nodev,noexec,relatime)
proc on /proc type proc (rw,nosuid,nodev,noexec,relatime)
udev on /dev type devtmpfs (rw,nosuid,relatime,size=8121732k,nr_inodes=2030433,mode=755)
devpts on /dev/pts type devpts (rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=000)
tmpfs on /run type tmpfs (rw,nosuid,noexec,relatime,size=1628440k,mode=755)
/dev/nvme0n1p2 on / type ext4 (rw,relatime,errors=remount-ro,data=ordered)
securityfs on /sys/kernel/security type securityfs (rw,nosuid,nodev,noexec,relatime)
tmpfs on /dev/shm type tmpfs (rw,nosuid,nodev)
tmpfs on /sys/fs/cgroup type tmpfs (ro,nosuid,nodev,noexec,mode=755)
cgroup on /sys/fs/cgroup/systemd type cgroup (rw,nosuid,nodev,noexec,relatime,xattr,release_agent=/lib/systemd/systemd-cgroups-agent,name=systemd)
pstore on /sys/fs/pstore type pstore (rw,nosuid,nodev,noexec,relatime)
efivarfs on /sys/firmware/efi/efivars type efivarfs (rw,nosuid,nodev,noexec,relatime)
cgroup on /sys/fs/cgroup/cpu,cpuacct type cgroup (rw,nosuid,nodev,noexec,relatime,cpu,cpuacct)
cgroup on /sys/fs/cgroup/freezer type cgroup (rw,nosuid,nodev,noexec,relatime,freezer)
cgroup on /sys/fs/cgroup/pids type cgroup (rw,nosuid,nodev,noexec,relatime,pids)
cgroup on /sys/fs/cgroup/cpuset type cgroup (rw,nosuid,nodev,noexec,relatime,cpuset)
systemd-1 on /proc/sys/fs/binfmt_misc type autofs (rw,relatime,fd=26,pgrp=1,timeout=0,minproto=5,maxproto=5,direct)
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime)
debugfs on /sys/kernel/debug type debugfs (rw,relatime)
mqueue on /dev/mqueue type mqueue (rw,relatime)
fusectl on /sys/fs/fuse/connections type fusectl (rw,relatime)
/dev/nvme0n1p1 on /boot/efi type vfat (rw,relatime,fmask=0077,dmask=0077,codepage=437,iocharset=iso8859-1,shortname=mixed,errors=remount-ro)
/dev/nvme0n1p2 on /var/lib/docker/aufs type ext4 (rw,relatime,errors=remount-ro,data=ordered)
gvfsd-fuse on /run/user/1002/gvfs type fuse.gvfsd-fuse (rw,nosuid,nodev,relatime,user_id=1002,group_id=1002)
''', 0, []),
# OS X, no CIFS
(r'''/dev/disk2 on / (hfs, local, journaled)
devfs on /dev (devfs, local, nobrowse)
map -hosts on /net (autofs, nosuid, automounted, nobrowse)
map auto_home on /home (autofs, automounted, nobrowse)
map -fstab on /Network/Servers (autofs, automounted, nobrowse)
/dev/disk3s2 on /Volumes/MyBookData (hfs, local, nodev, nosuid, journaled)
afni:/elrond0 on /Volumes/afni (nfs)
afni:/var/www/INCOMING on /Volumes/INCOMING (nfs)
afni:/fraid on /Volumes/afni (nfs, asynchronous)
boromir:/raid.bot on /Volumes/raid.bot (nfs)
elros:/volume2/AFNI_SHARE on /Volumes/AFNI_SHARE (nfs)
map -static on /Volumes/safni (autofs, automounted, nobrowse)
map -static on /Volumes/raid.top (autofs, automounted, nobrowse)
/dev/disk1s3 on /Volumes/Boot OS X (hfs, local, journaled, nobrowse)
''', 0, []),
# Non-zero exit code
('', 1, []),
# Variant of Linux example with CIFS added manually
(r'''sysfs on /sys type sysfs (rw,nosuid,nodev,noexec,relatime)
proc on /proc type proc (rw,nosuid,nodev,noexec,relatime)
udev on /dev type devtmpfs (rw,nosuid,relatime,size=8121732k,nr_inodes=2030433,mode=755)
devpts on /dev/pts type devpts (rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=000)
tmpfs on /run type tmpfs (rw,nosuid,noexec,relatime,size=1628440k,mode=755)
/dev/nvme0n1p2 on / type ext4 (rw,relatime,errors=remount-ro,data=ordered)
securityfs on /sys/kernel/security type securityfs (rw,nosuid,nodev,noexec,relatime)
tmpfs on /dev/shm type tmpfs (rw,nosuid,nodev)
tmpfs on /sys/fs/cgroup type tmpfs (ro,nosuid,nodev,noexec,mode=755)
cgroup on /sys/fs/cgroup/systemd type cgroup (rw,nosuid,nodev,noexec,relatime,xattr,release_agent=/lib/systemd/systemd-cgroups-agent,name=systemd)
pstore on /sys/fs/pstore type pstore (rw,nosuid,nodev,noexec,relatime)
efivarfs on /sys/firmware/efi/efivars type efivarfs (rw,nosuid,nodev,noexec,relatime)
cgroup on /sys/fs/cgroup/cpu,cpuacct type cgroup (rw,nosuid,nodev,noexec,relatime,cpu,cpuacct)
cgroup on /sys/fs/cgroup/freezer type cgroup (rw,nosuid,nodev,noexec,relatime,freezer)
cgroup on /sys/fs/cgroup/pids type cgroup (rw,nosuid,nodev,noexec,relatime,pids)
cgroup on /sys/fs/cgroup/cpuset type cgroup (rw,nosuid,nodev,noexec,relatime,cpuset)
systemd-1 on /proc/sys/fs/binfmt_misc type autofs (rw,relatime,fd=26,pgrp=1,timeout=0,minproto=5,maxproto=5,direct)
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime)
debugfs on /sys/kernel/debug type debugfs (rw,relatime)
mqueue on /dev/mqueue type mqueue (rw,relatime)
fusectl on /sys/fs/fuse/connections type fusectl (rw,relatime)
/dev/nvme0n1p1 on /boot/efi type vfat (rw,relatime,fmask=0077,dmask=0077,codepage=437,iocharset=iso8859-1,shortname=mixed,errors=remount-ro)
/dev/nvme0n1p2 on /var/lib/docker/aufs type ext4 (rw,relatime,errors=remount-ro,data=ordered)
gvfsd-fuse on /run/user/1002/gvfs type fuse.gvfsd-fuse (rw,nosuid,nodev,relatime,user_id=1002,group_id=1002)
''', 0, []),
# Variant of OS X example with CIFS added manually
(r'''/dev/disk2 on / (hfs, local, journaled)
devfs on /dev (devfs, local, nobrowse)
afni:/elrond0 on /Volumes/afni (cifs)
afni:/var/www/INCOMING on /Volumes/INCOMING (nfs)
afni:/fraid on /Volumes/afni/fraid (nfs, asynchronous)
boromir:/raid.bot on /Volumes/raid.bot (nfs)
elros:/volume2/AFNI_SHARE on /Volumes/AFNI_SHARE (nfs)
''', 0, [('/Volumes/afni/fraid', 'nfs'), ('/Volumes/afni', 'cifs')]),
# From Windows: docker run --rm -it -v C:\:/data busybox mount
(r'''overlay on / type overlay (rw,relatime,lowerdir=/var/lib/docker/overlay2/l/26UTYITLF24YE7KEGTMHUNHPPG:/var/lib/docker/overlay2/l/SWGNP3T2EEB4CNBJFN3SDZLXHP,upperdir=/var/lib/docker/overlay2/a4c54ab1aa031bb5a14a424abd655510521e183ee4fa4158672e8376c89df394/diff,workdir=/var/lib/docker/overlay2/a4c54ab1aa031bb5a14a424abd655510521e183ee4fa4158672e8376c89df394/work)
proc on /proc type proc (rw,nosuid,nodev,noexec,relatime)
tmpfs on /dev type tmpfs (rw,nosuid,size=65536k,mode=755)
devpts on /dev/pts type devpts (rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=666)
sysfs on /sys type sysfs (ro,nosuid,nodev,noexec,relatime)
tmpfs on /sys/fs/cgroup type tmpfs (ro,nosuid,nodev,noexec,relatime,mode=755)
cpuset on /sys/fs/cgroup/cpuset type cgroup (ro,nosuid,nodev,noexec,relatime,cpuset)
cpu on /sys/fs/cgroup/cpu type cgroup (ro,nosuid,nodev,noexec,relatime,cpu)
cpuacct on /sys/fs/cgroup/cpuacct type cgroup (ro,nosuid,nodev,noexec,relatime,cpuacct)
blkio on /sys/fs/cgroup/blkio type cgroup (ro,nosuid,nodev,noexec,relatime,blkio)
memory on /sys/fs/cgroup/memory type cgroup (ro,nosuid,nodev,noexec,relatime,memory)
devices on /sys/fs/cgroup/devices type cgroup (ro,nosuid,nodev,noexec,relatime,devices)
freezer on /sys/fs/cgroup/freezer type cgroup (ro,nosuid,nodev,noexec,relatime,freezer)
net_cls on /sys/fs/cgroup/net_cls type cgroup (ro,nosuid,nodev,noexec,relatime,net_cls)
perf_event on /sys/fs/cgroup/perf_event type cgroup (ro,nosuid,nodev,noexec,relatime,perf_event)
net_prio on /sys/fs/cgroup/net_prio type cgroup (ro,nosuid,nodev,noexec,relatime,net_prio)
hugetlb on /sys/fs/cgroup/hugetlb type cgroup (ro,nosuid,nodev,noexec,relatime,hugetlb)
pids on /sys/fs/cgroup/pids type cgroup (ro,nosuid,nodev,noexec,relatime,pids)
cgroup on /sys/fs/cgroup/systemd type cgroup (ro,nosuid,nodev,noexec,relatime,name=systemd)
mqueue on /dev/mqueue type mqueue (rw,nosuid,nodev,noexec,relatime)
//10.0.75.1/C on /data type cifs (rw,relatime,vers=3.02,sec=ntlmsspi,cache=strict,username=filo,domain=MSI,uid=0,noforceuid,gid=0,noforcegid,addr=10.0.75.1,file_mode=0755,dir_mode=0755,iocharset=utf8,nounix,serverino,mapposix,nobrl,mfsymlinks,noperm,rsize=1048576,wsize=1048576,echo_interval=60,actimeo=1)
/dev/sda1 on /etc/resolv.conf type ext4 (rw,relatime,data=ordered)
/dev/sda1 on /etc/hostname type ext4 (rw,relatime,data=ordered)
/dev/sda1 on /etc/hosts type ext4 (rw,relatime,data=ordered)
shm on /dev/shm type tmpfs (rw,nosuid,nodev,noexec,relatime,size=65536k)
devpts on /dev/console type devpts (rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=666)
proc on /proc/bus type proc (ro,relatime)
proc on /proc/fs type proc (ro,relatime)
proc on /proc/irq type proc (ro,relatime)
proc on /proc/sys type proc (ro,relatime)
proc on /proc/sysrq-trigger type proc (ro,relatime)
tmpfs on /proc/kcore type tmpfs (rw,nosuid,size=65536k,mode=755)
tmpfs on /proc/timer_list type tmpfs (rw,nosuid,size=65536k,mode=755)
tmpfs on /proc/sched_debug type tmpfs (rw,nosuid,size=65536k,mode=755)
tmpfs on /proc/scsi type tmpfs (ro,relatime)
tmpfs on /sys/firmware type tmpfs (ro,relatime)
''', 0, [('/data', 'cifs')]),
# From @yarikoptic - added blank lines to test for resilience
(r'''/proc on /proc type proc (rw,relatime)
sysfs on /sys type sysfs (rw,nosuid,nodev,noexec,relatime)
tmpfs on /dev/shm type tmpfs (rw,relatime)
devpts on /dev/pts type devpts (rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=666)

devpts on /dev/ptmx type devpts (rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=666)

''', 0, []),
)


@pytest.mark.parametrize("output, exit_code, expected", MOUNT_OUTPUTS)
def test_parse_mount_table(output, exit_code, expected):
    assert _parse_mount_table(exit_code, output) == expected


def test_cifs_check():
    assert isinstance(_cifs_table, list)
    assert isinstance(on_cifs('/'), bool)
    fake_table = [('/scratch/tmp', 'ext4'), ('/scratch', 'cifs')]
    cifs_targets = [('/scratch/tmp/x/y', False), ('/scratch/tmp/x', False),
                    ('/scratch/x/y', True), ('/scratch/x', True),
                    ('/x/y', False), ('/x', False), ('/', False)]

    orig_table = _cifs_table[:]
    _cifs_table[:] = []

    for target, _ in cifs_targets:
        assert on_cifs(target) is False

    _cifs_table.extend(fake_table)
    for target, expected in cifs_targets:
        assert on_cifs(target) is expected

    _cifs_table[:] = []
    _cifs_table.extend(orig_table)


def test_indirectory(tmpdir):
    tmpdir.chdir()

    os.makedirs('subdir1/subdir2')
    sd1 = os.path.abspath('subdir1')
    sd2 = os.path.abspath('subdir1/subdir2')

    assert os.getcwd() == tmpdir.strpath
    with indirectory('/'):
        assert os.getcwd() == '/'
    assert os.getcwd() == tmpdir.strpath
    with indirectory('subdir1'):
        assert os.getcwd() == sd1
        with indirectory('subdir2'):
            assert os.getcwd() == sd2
            with indirectory('..'):
                assert os.getcwd() == sd1
                with indirectory('/'):
                    assert os.getcwd() == '/'
                assert os.getcwd() == sd1
            assert os.getcwd() == sd2
        assert os.getcwd() == sd1
    assert os.getcwd() == tmpdir.strpath
    try:
        with indirectory('subdir1'):
            raise ValueError("Erroring out of context")
    except ValueError:
        pass
    assert os.getcwd() == tmpdir.strpath



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..base import SEMLikeCommandLine


def test_SEMLikeCommandLine_inputs():
    input_map = dict(args=dict(argstr='%s',
    ),
    environ=dict(nohash=True,
    usedefault=True,
    ),
    ignore_exception=dict(nohash=True,
    usedefault=True,
    ),
    terminal_output=dict(deprecated='1.0.0',
    nohash=True,
    ),
    )
    inputs = SEMLikeCommandLine.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value




# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import
import os

import pytest
import nipype.interfaces.fsl as fsl
import nipype.interfaces.utility as util
from nipype.interfaces.fsl import no_fsl, no_fsl_course_data

import nipype.pipeline.engine as pe
import warnings
from nipype.workflows.dmri.fsl.dti import create_bedpostx_pipeline
from nipype.utils.filemanip import simplify_list


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
@pytest.mark.skipif(no_fsl_course_data(), reason="fsl data not available")
def test_create_bedpostx_pipeline(tmpdir):
    fsl_course_dir = os.path.abspath(os.environ['FSL_COURSE_DATA'])

    mask_file = os.path.join(fsl_course_dir,
                             "fdt2/subj1.bedpostX/nodif_brain_mask.nii.gz")
    bvecs_file = os.path.join(fsl_course_dir, "fdt2/subj1/bvecs")
    bvals_file = os.path.join(fsl_course_dir, "fdt2/subj1/bvals")
    dwi_file = os.path.join(fsl_course_dir, "fdt2/subj1/data.nii.gz")
    z_min = 62
    z_size = 2

    slice_mask = pe.Node(
        fsl.ExtractROI(
            x_min=0, x_size=-1, y_min=0, y_size=-1, z_min=z_min,
            z_size=z_size),
        name="slice_mask")
    slice_mask.inputs.in_file = mask_file

    slice_dwi = pe.Node(
        fsl.ExtractROI(
            x_min=0, x_size=-1, y_min=0, y_size=-1, z_min=z_min,
            z_size=z_size),
        name="slice_dwi")
    slice_dwi.inputs.in_file = dwi_file

    nipype_bedpostx = create_bedpostx_pipeline("nipype_bedpostx")
    nipype_bedpostx.inputs.inputnode.bvecs = bvecs_file
    nipype_bedpostx.inputs.inputnode.bvals = bvals_file
    nipype_bedpostx.inputs.xfibres.n_fibres = 1
    nipype_bedpostx.inputs.xfibres.fudge = 1
    nipype_bedpostx.inputs.xfibres.burn_in = 0
    nipype_bedpostx.inputs.xfibres.n_jumps = 1
    nipype_bedpostx.inputs.xfibres.sample_every = 1
    nipype_bedpostx.inputs.xfibres.cnlinear = True
    nipype_bedpostx.inputs.xfibres.seed = 0
    nipype_bedpostx.inputs.xfibres.model = 2

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        original_bedpostx = pe.Node(
            interface=fsl.BEDPOSTX(), name="original_bedpostx")
    original_bedpostx.inputs.bvecs = bvecs_file
    original_bedpostx.inputs.bvals = bvals_file
    original_bedpostx.inputs.environ['FSLPARALLEL'] = ""
    original_bedpostx.inputs.n_fibres = 1
    original_bedpostx.inputs.fudge = 1
    original_bedpostx.inputs.burn_in = 0
    original_bedpostx.inputs.n_jumps = 1
    original_bedpostx.inputs.sample_every = 1
    original_bedpostx.inputs.seed = 0
    original_bedpostx.inputs.model = 2

    test_f1 = pe.Node(util.AssertEqual(), name="mean_f1_test")

    pipeline = pe.Workflow(name="test_bedpostx")
    pipeline.base_dir = tmpdir.mkdir("nipype_test_bedpostx_").strpath

    pipeline.connect([
        (slice_mask, original_bedpostx, [("roi_file", "mask")]),
        (slice_mask, nipype_bedpostx, [("roi_file", "inputnode.mask")]),
        (slice_dwi, original_bedpostx, [("roi_file", "dwi")]),
        (slice_dwi, nipype_bedpostx, [("roi_file", "inputnode.dwi")]),
        (nipype_bedpostx, test_f1, [(("outputnode.mean_fsamples",
                                      simplify_list), "volume1")]),
        (original_bedpostx, test_f1, [("mean_fsamples", "volume2")]),
    ])

    pipeline.run(plugin='Linear')



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..base import Split


def test_Split_inputs():
    input_map = dict(
        ignore_exception=dict(
            deprecated='1.0.0',
            nohash=True,
            usedefault=True,
        ),
        inlist=dict(mandatory=True, ),
        splits=dict(mandatory=True, ),
        squeeze=dict(usedefault=True, ),
    )
    inputs = Split.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Split_outputs():
    output_map = dict()
    outputs = Split.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..io import XNATSource


def test_XNATSource_inputs():
    input_map = dict(
        cache_dir=dict(),
        config=dict(
            mandatory=True,
            xor=['server'],
        ),
        ignore_exception=dict(
            deprecated='1.0.0',
            nohash=True,
            usedefault=True,
        ),
        pwd=dict(),
        query_template=dict(mandatory=True, ),
        query_template_args=dict(usedefault=True, ),
        server=dict(
            mandatory=True,
            requires=['user', 'pwd'],
            xor=['config'],
        ),
        user=dict(),
    )
    inputs = XNATSource.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_XNATSource_outputs():
    output_map = dict()
    outputs = XNATSource.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
